@@ -21,7 +21,6 @@
 //!
 //! intercept:
 //!   mode: wrapper
-//!   directory: /tmp
 //!
 //! compilers:
 //!   - path: /usr/local/bin/cc
@@ -31,7 +30,6 @@
 //!   - path: /usr/bin/clang++
 //!
 //! sources:
-//!   only_existing_files: true
 //!   directories:
 //!     - path: "/opt/project/sources"
 //!       action: include
@@ -122,7 +120,7 @@ mod types {
         }
     }
 
-    /// Simplified intercept configuration with mode and directory.
+    /// Simplified intercept configuration with mode.
     #[derive(Debug, PartialEq, serde::Deserialize, serde::Serialize)]
     #[serde(tag = "mode")]
     pub enum Intercept {
@@ -130,8 +128,6 @@ mod types {
         Wrapper {
             #[serde(default = "default_wrapper_executable")]
             path: PathBuf,
-            #[serde(default = "default_wrapper_directory")]
-            directory: PathBuf,
         },
         #[serde(rename = "preload")]
         Preload {
@@ -142,20 +138,14 @@ mod types {
 
     /// The default intercept mode is varying based on the target operating system.
     impl Default for Intercept {
-        #[cfg(any(
-            target_os = "linux",
-            target_os = "freebsd",
-            target_os = "netbsd",
-            target_os = "openbsd",
-            target_os = "dragonfly"
-        ))]
+        #[cfg(not(any(target_os = "macos", target_os = "ios", target_os = "windows")))]
         fn default() -> Self {
             Intercept::Preload { path: default_preload_library() }
         }
 
         #[cfg(any(target_os = "macos", target_os = "ios", target_os = "windows"))]
         fn default() -> Self {
-            Intercept::Wrapper { path: default_wrapper_executable(), directory: default_wrapper_directory() }
+            Intercept::Wrapper { path: default_wrapper_executable() }
         }
     }
 
@@ -236,20 +226,12 @@ mod types {
     ///
     /// **Important**: For matching to work correctly, rule paths should use the same format as
     /// configured in `format.paths.file`. This consistency is the user's responsibility.
-    #[derive(Clone, Debug, PartialEq, serde::Deserialize, serde::Serialize)]
+    #[derive(Clone, Debug, Default, PartialEq, serde::Deserialize, serde::Serialize)]
     pub struct SourceFilter {
-        #[serde(default = "default_enabled")]
-        pub only_existing_files: bool,
         #[serde(default, skip_serializing_if = "Vec::is_empty")]
         pub directories: Vec<DirectoryRule>,
     }
 
-    impl Default for SourceFilter {
-        fn default() -> Self {
-            Self { only_existing_files: true, directories: vec![] }
-        }
-    }
-
     /// Duplicate filter configuration matching the YAML format.
     #[derive(Clone, Debug, PartialEq, serde::Deserialize, serde::Serialize)]
     pub struct DuplicateFilter {
@@ -258,7 +240,7 @@ mod types {
 
     impl Default for DuplicateFilter {
         fn default() -> Self {
-            Self { match_on: vec![OutputFields::File, OutputFields::Arguments] }
+            Self { match_on: vec![OutputFields::Directory, OutputFields::File, OutputFields::Arguments] }
         }
     }
 
@@ -332,11 +314,6 @@ mod types {
     const PRELOAD_LIBRARY_PATH: &str = env!("PRELOAD_LIBRARY_PATH");
     const WRAPPER_EXECUTABLE_PATH: &str = env!("WRAPPER_EXECUTABLE_PATH");
 
-    /// The default directory where the wrapper executables will be stored.
-    pub(super) fn default_wrapper_directory() -> PathBuf {
-        std::env::temp_dir()
-    }
-
     /// The default path to the wrapper executable.
     pub(super) fn default_wrapper_executable() -> PathBuf {
         PathBuf::from(WRAPPER_EXECUTABLE_PATH)
@@ -389,6 +366,8 @@ pub mod validation {
         PathNotFound { path: String },
         #[error("Duplicate {field} entry at: {idx}")]
         DuplicateEntry { field: &'static str, idx: usize },
+        #[error("Path format error: {message}")]
+        PathFormatError { message: &'static str },
         #[error("Multiple validation errors: {errors:?}")]
         Multiple { errors: Vec<ValidationError> },
     }
@@ -458,6 +437,9 @@ pub mod validation {
             // Validate duplicate filter configuration
             collector.add_result(DuplicateFilter::validate(&config.duplicates));
 
+            // Validate path format configuration
+            collector.add_result(PathFormat::validate(&config.format.paths));
+
             collector.finish()
         }
     }
@@ -467,19 +449,12 @@ pub mod validation {
 
         fn validate(config: &Intercept) -> Result<(), Self::Error> {
             match config {
-                Intercept::Wrapper { path, directory } => {
-                    let mut collector = ValidationCollector::new();
-
+                Intercept::Wrapper { path } => {
                     if !path.exists() {
-                        collector.add(ValidationError::PathNotFound { path: path.display().to_string() });
+                        Err(ValidationError::PathNotFound { path: path.display().to_string() })
+                    } else {
+                        Ok(())
                     }
-
-                    if !directory.exists() {
-                        collector
-                            .add(ValidationError::PathNotFound { path: directory.display().to_string() });
-                    }
-
-                    collector.finish()
                 }
                 Intercept::Preload { path } => {
                     if !path.exists() {
@@ -544,6 +519,34 @@ pub mod validation {
         }
     }
 
+    impl Validator<PathFormat> for PathFormat {
+        type Error = ValidationError;
+
+        /// Validates the path format configuration according to the rules:
+        /// - When directory is relative, file must be relative too
+        /// - When directory is canonical, file can't be absolute
+        /// - When directory is absolute, file can't be canonical
+        fn validate(config: &PathFormat) -> Result<(), Self::Error> {
+            use PathResolver::*;
+
+            match (&config.directory, &config.file) {
+                (Relative, Absolute | Canonical) => Err(ValidationError::PathFormatError {
+                    message: "When directory is relative, file must be relative too",
+                }),
+                (Canonical, Absolute) => Err(ValidationError::PathFormatError {
+                    message: "When directory is canonical, file can't be absolute",
+                }),
+                (Absolute, Canonical) => Err(ValidationError::PathFormatError {
+                    message: "When directory is absolute, file can't be canonical",
+                }),
+                (AsIs, Absolute | Relative | Canonical) => Err(ValidationError::PathFormatError {
+                    message: "When directory as-is, file should be the same",
+                }),
+                _ => Ok(()),
+            }
+        }
+    }
+
     #[cfg(test)]
     mod tests {
         use super::*;
@@ -556,26 +559,21 @@ pub mod validation {
             let temp_file = temp_dir.path().join("test_file");
             std::fs::write(&temp_file, "test").unwrap();
 
-            let config = Intercept::Wrapper { path: temp_file, directory: temp_dir.path().to_path_buf() };
+            let config = Intercept::Wrapper { path: temp_file };
 
             assert!(Intercept::validate(&config).is_ok());
         }
 
         #[test]
         fn test_validate_intercept_wrapper_invalid_paths() {
-            let config = Intercept::Wrapper {
-                path: PathBuf::from("/nonexistent/path"),
-                directory: PathBuf::from("/nonexistent/directory"),
-            };
+            let config = Intercept::Wrapper { path: PathBuf::from("/nonexistent/path") };
 
             let result = Intercept::validate(&config);
             assert!(result.is_err());
 
             match result.unwrap_err() {
-                ValidationError::Multiple { errors } => {
-                    assert_eq!(errors.len(), 2);
-                }
-                _ => panic!("Expected multiple validation errors"),
+                ValidationError::PathNotFound { .. } => {}
+                _ => panic!("Expected PathNotFound validation error"),
             }
         }
 
@@ -601,7 +599,6 @@ pub mod validation {
         #[test]
         fn test_validate_source_filter_empty_paths() {
             let config = SourceFilter {
-                only_existing_files: true,
                 directories: vec![
                     DirectoryRule { path: PathBuf::from("valid/path"), action: DirectoryAction::Include },
                     DirectoryRule { path: PathBuf::from(""), action: DirectoryAction::Exclude },
@@ -622,7 +619,6 @@ pub mod validation {
         #[test]
         fn test_validate_source_filter_multiple_empty_paths() {
             let config = SourceFilter {
-                only_existing_files: true,
                 directories: vec![
                     DirectoryRule { path: PathBuf::from(""), action: DirectoryAction::Include },
                     DirectoryRule { path: PathBuf::from("valid/path"), action: DirectoryAction::Exclude },
@@ -644,7 +640,6 @@ pub mod validation {
         #[test]
         fn test_validate_source_filter_valid_config() {
             let config = SourceFilter {
-                only_existing_files: true,
                 directories: vec![
                     DirectoryRule { path: PathBuf::from("/usr/include"), action: DirectoryAction::Exclude },
                     DirectoryRule { path: PathBuf::from("src"), action: DirectoryAction::Include },
@@ -657,7 +652,7 @@ pub mod validation {
 
         #[test]
         fn test_validate_source_filter_empty_directories() {
-            let config = SourceFilter { only_existing_files: false, directories: vec![] };
+            let config = SourceFilter { directories: vec![] };
 
             let result = SourceFilter::validate(&config);
             assert!(result.is_ok());
@@ -721,6 +716,57 @@ pub mod validation {
             let result = DuplicateFilter::validate(&config);
             assert!(result.is_ok());
         }
+
+        #[test]
+        fn test_validate_path_format_success() {
+            let valid_configs = vec![
+                PathFormat { directory: PathResolver::AsIs, file: PathResolver::AsIs },
+                PathFormat { directory: PathResolver::Relative, file: PathResolver::Relative },
+                PathFormat { directory: PathResolver::Canonical, file: PathResolver::Relative },
+                PathFormat { directory: PathResolver::Absolute, file: PathResolver::Relative },
+                PathFormat { directory: PathResolver::Absolute, file: PathResolver::Absolute },
+            ];
+
+            for config in valid_configs {
+                assert!(PathFormat::validate(&config).is_ok(), "Config should be valid: {:?}", config);
+            }
+        }
+
+        #[test]
+        fn test_validate_path_format_failures() {
+            let invalid_configs = vec![
+                (
+                    PathFormat { directory: PathResolver::Relative, file: PathResolver::Absolute },
+                    "When directory is relative, file must be relative too",
+                ),
+                (
+                    PathFormat { directory: PathResolver::Relative, file: PathResolver::Canonical },
+                    "When directory is relative, file must be relative too",
+                ),
+                (
+                    PathFormat { directory: PathResolver::Canonical, file: PathResolver::Absolute },
+                    "When directory is canonical, file can't be absolute",
+                ),
+                (
+                    PathFormat { directory: PathResolver::Absolute, file: PathResolver::Canonical },
+                    "When directory is absolute, file can't be canonical",
+                ),
+                (
+                    PathFormat { directory: PathResolver::AsIs, file: PathResolver::Canonical },
+                    "When directory as-is, file should be the same",
+                ),
+            ];
+
+            for (config, expected_error) in invalid_configs {
+                let result = PathFormat::validate(&config);
+                assert!(result.is_err(), "Config should be invalid: {:?}", config);
+                if let Err(ValidationError::PathFormatError { message }) = result {
+                    assert_eq!(message, expected_error);
+                } else {
+                    panic!("Expected PathFormatError, got: {:?}", result);
+                }
+            }
+        }
     }
 }
 
@@ -862,7 +908,6 @@ pub mod loader {
             intercept:
                 mode: wrapper
                 path: /usr/local/libexec/bear/wrapper
-                directory: /tmp
 
             compilers:
               - path: /usr/local/bin/cc
@@ -875,7 +920,6 @@ pub mod loader {
                     remove: ["-Wall"]
 
             sources:
-                only_existing_files: true
                 directories:
                   - path: "/opt/project/sources"
                     action: include
@@ -898,10 +942,7 @@ pub mod loader {
 
             let expected = Main {
                 schema: String::from("4.0"),
-                intercept: Intercept::Wrapper {
-                    path: PathBuf::from("/usr/local/libexec/bear/wrapper"),
-                    directory: PathBuf::from("/tmp"),
-                },
+                intercept: Intercept::Wrapper { path: PathBuf::from("/usr/local/libexec/bear/wrapper") },
                 compilers: vec![
                     Compiler {
                         path: PathBuf::from("/usr/local/bin/cc"),
@@ -912,7 +953,6 @@ pub mod loader {
                     Compiler { path: PathBuf::from("/usr/bin/clang++"), as_: None, ignore: false },
                 ],
                 sources: SourceFilter {
-                    only_existing_files: true,
                     directories: vec![
                         DirectoryRule {
                             path: PathBuf::from("/opt/project/sources"),
@@ -952,13 +992,12 @@ pub mod loader {
 
             let expected = Main {
                 schema: String::from("4.0"),
-                intercept: Intercept::Wrapper {
-                    path: default_wrapper_executable(),
-                    directory: default_wrapper_directory(),
-                },
+                intercept: Intercept::Wrapper { path: default_wrapper_executable() },
                 compilers: vec![],
-                sources: SourceFilter { only_existing_files: true, directories: vec![] },
-                duplicates: DuplicateFilter { match_on: vec![OutputFields::File, OutputFields::Arguments] },
+                sources: SourceFilter { directories: vec![] },
+                duplicates: DuplicateFilter {
+                    match_on: vec![OutputFields::Directory, OutputFields::File, OutputFields::Arguments],
+                },
                 format: Format {
                     paths: PathFormat { directory: PathResolver::AsIs, file: PathResolver::AsIs },
                     entries: EntryFormat { use_array_format: true, include_output_field: true },
@@ -975,8 +1014,6 @@ pub mod loader {
 
             intercept:
               mode: preload
-            sources:
-              only_existing_files: false
             format:
               paths:
                 directory: absolute
@@ -989,8 +1026,10 @@ pub mod loader {
                 schema: String::from("4.0"),
                 intercept: Intercept::Preload { path: default_preload_library() },
                 compilers: vec![],
-                sources: SourceFilter { only_existing_files: false, directories: vec![] },
-                duplicates: DuplicateFilter { match_on: vec![OutputFields::File, OutputFields::Arguments] },
+                sources: SourceFilter { directories: vec![] },
+                duplicates: DuplicateFilter {
+                    match_on: vec![OutputFields::Directory, OutputFields::File, OutputFields::Arguments],
+                },
                 format: Format {
                     paths: PathFormat { directory: PathResolver::Absolute, file: PathResolver::Absolute },
                     entries: EntryFormat { use_array_format: true, include_output_field: true },
