@@ -8,34 +8,81 @@
 /// The filename used for wrapper configuration files.
 pub const CONFIG_FILENAME: &str = "wrappers.cfg";
 
+/// The directory name used for wrapper executables in the current working directory.
+pub const WRAPPER_DIR_NAME: &str = ".bear";
+
 use std::collections::HashMap;
 use std::fmt::Display;
 use std::io::{BufReader, BufWriter, Read, Write};
+use std::net::SocketAddr;
 use std::path::{Path, PathBuf};
-use tempfile::TempDir;
 use thiserror::Error;
 
+/// A managed directory that is created in the current working directory
+/// and cleaned up when dropped.
+///
+/// Unlike `TempDir`, this uses a deterministic path (`.bear/` in cwd) which
+/// is essential for autotools-style builds where `configure` caches compiler
+/// paths that must remain valid across multiple Bear invocations.
+pub struct ManagedDirectory {
+    path: PathBuf,
+}
+
+impl ManagedDirectory {
+    /// Creates a new managed directory at `.bear/` in the specified working directory.
+    ///
+    /// If the directory already exists, it is deleted first to ensure a clean state.
+    /// This is important because we need fresh wrapper executables for each Bear run.
+    ///
+    /// # Arguments
+    ///
+    /// * `working_dir` - The directory where `.bear/` will be created (typically from context)
+    pub fn create(working_dir: &Path) -> Result<Self, std::io::Error> {
+        let path = working_dir.join(WRAPPER_DIR_NAME);
+
+        // Remove existing directory if present (clean slate for each run)
+        if path.exists() {
+            std::fs::remove_dir_all(&path)?;
+        }
+
+        // Create fresh directory
+        std::fs::create_dir_all(&path)?;
+
+        Ok(Self { path })
+    }
+
+    /// Returns the path to the managed directory.
+    pub fn path(&self) -> &Path {
+        &self.path
+    }
+}
+
+impl Drop for ManagedDirectory {
+    fn drop(&mut self) {
+        // Best-effort cleanup - ignore errors
+        if self.path.exists() {
+            let _ = std::fs::remove_dir_all(&self.path);
+        }
+    }
+}
+
 /// Configuration structure for wrapper executables.
 ///
 /// This structure contains the mapping from wrapper executable names to their
-/// corresponding real executable paths. It is designed to be minimal and focused
-/// solely on the executable mappings.
+/// corresponding real executable paths, as well as the address of the collector
+/// where intercepted events should be reported.
 #[derive(Clone, Debug, PartialEq, serde::Serialize, serde::Deserialize)]
 pub struct WrapperConfig {
     /// Map from wrapper executable name to the real executable path
     pub executables: HashMap<String, PathBuf>,
-}
-
-impl Default for WrapperConfig {
-    fn default() -> Self {
-        Self::new()
-    }
+    /// The address of the collector to report intercepted events to
+    pub collector_address: SocketAddr,
 }
 
 impl WrapperConfig {
-    /// Creates a new empty wrapper configuration.
-    pub fn new() -> Self {
-        Self { executables: HashMap::new() }
+    /// Creates a new wrapper configuration with the specified collector address.
+    pub fn new(collector_address: SocketAddr) -> Self {
+        Self { executables: HashMap::new(), collector_address }
     }
 
     /// Adds an executable mapping to the configuration.
@@ -51,6 +98,7 @@ impl WrapperConfig {
 
 impl Display for WrapperConfig {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        writeln!(f, "Collector address: {}", self.collector_address)?;
         if self.executables.is_empty() {
             write!(f, "No executables configured")
         } else {
@@ -104,33 +152,41 @@ impl WrapperConfigWriter {
 ///
 /// This type acts as a builder that collects executable mappings and creates
 /// the necessary directory structure, hard links (or file copies on Windows), and configuration file.
-/// It owns and manages the temporary directory for the wrapper setup.
-/// Builder for creating wrapper directories with executable links and configuration.
+/// It owns and manages the wrapper directory (`.bear/` in the current working directory).
 ///
-/// This type acts as a builder that collects executable mappings and creates
-/// the necessary directory structure, hard links (or file copies on Windows), and configuration file.
-/// It owns and manages the temporary directory for the wrapper setup.
+/// The use of a deterministic directory path is essential for autotools-style builds
+/// where `./configure` caches compiler paths. Using a random temporary directory
+/// would break subsequent `make` invocations because the cached paths would no longer exist.
 pub struct WrapperDirectoryBuilder {
     wrapper_executable_path: PathBuf,
-    wrapper_dir: TempDir,
+    wrapper_dir: ManagedDirectory,
     config: WrapperConfig,
 }
 
 impl WrapperDirectoryBuilder {
-    /// Creates a wrapper directory in a temporary location and sets it up.
+    /// Creates a wrapper directory at `.bear/` in the specified working directory.
     ///
-    /// This is a convenience method that combines directory creation with setup.
-    /// The WrapperDirectoryBuilder takes ownership of the temporary directory.
-    pub fn create(wrapper_executable_path: &Path, parent_dir: &Path) -> Result<Self, WrapperDirectoryError> {
-        let wrapper_dir = tempfile::Builder::new()
-            .prefix("bear-")
-            .tempdir_in(parent_dir)
-            .map_err(WrapperDirectoryError::TempDirCreation)?;
+    /// This creates a deterministic directory path that persists across Bear invocations,
+    /// which is essential for autotools-style builds. The directory is cleaned up when
+    /// Bear exits, but the path remains consistent so cached compiler paths remain valid.
+    ///
+    /// # Arguments
+    ///
+    /// * `wrapper_executable_path` - Path to the wrapper executable to link/copy
+    /// * `working_dir` - The directory where `.bear/` will be created (typically `context.current_directory`)
+    /// * `collector_address` - The address of the collector to report intercepted events to
+    pub fn create(
+        wrapper_executable_path: &Path,
+        working_dir: &Path,
+        collector_address: SocketAddr,
+    ) -> Result<Self, WrapperDirectoryError> {
+        let wrapper_dir =
+            ManagedDirectory::create(working_dir).map_err(WrapperDirectoryError::DirCreation)?;
 
         Ok(Self {
             wrapper_executable_path: wrapper_executable_path.to_path_buf(),
             wrapper_dir,
-            config: WrapperConfig::new(),
+            config: WrapperConfig::new(collector_address),
         })
     }
 
@@ -196,7 +252,7 @@ impl WrapperDirectoryBuilder {
 
 /// Immutable wrapper directory after build.
 pub struct WrapperDirectory {
-    wrapper_dir: TempDir,
+    wrapper_dir: ManagedDirectory,
     _config: WrapperConfig,
 }
 
@@ -237,8 +293,8 @@ pub enum WrapperDirectoryError {
     LinkCreation(#[from] std::io::Error),
     #[error("Failed to write configuration file: {0}")]
     ConfigWrite(#[from] ConfigError),
-    #[error("Failed to create temporary directory: {0}")]
-    TempDirCreation(std::io::Error),
+    #[error("Failed to create wrapper directory: {0}")]
+    DirCreation(std::io::Error),
 }
 
 /// Errors that can occur during wrapper configuration operations.
@@ -254,18 +310,28 @@ pub enum ConfigError {
 mod tests {
     use super::*;
     use std::io::Cursor;
+    use std::net::{IpAddr, Ipv4Addr};
     use tempfile::TempDir;
 
+    fn address() -> SocketAddr {
+        SocketAddr::new(IpAddr::V4(Ipv4Addr::new(127, 0, 0, 1)), 8080)
+    }
+
     #[test]
     fn test_wrapper_config_new() {
-        let config = WrapperConfig::new();
+        let config = WrapperConfig::new(address());
+
         assert!(config.executables.is_empty());
+        assert_eq!(config.collector_address, address());
     }
 
     #[test]
     fn test_wrapper_config_add_executable() {
-        let mut config = WrapperConfig::new();
-        config.add_executable("gcc".to_string(), PathBuf::from("/usr/bin/gcc"));
+        let config = {
+            let mut builder = WrapperConfig::new(address());
+            builder.add_executable("gcc".to_string(), PathBuf::from("/usr/bin/gcc"));
+            builder
+        };
 
         assert_eq!(config.executables.len(), 1);
         assert_eq!(config.get_executable("gcc"), Some(&PathBuf::from("/usr/bin/gcc")));
@@ -273,9 +339,12 @@ mod tests {
 
     #[test]
     fn test_wrapper_config_get_executable() {
-        let mut config = WrapperConfig::new();
-        config.add_executable("gcc".to_string(), PathBuf::from("/usr/bin/gcc"));
-        config.add_executable("g++".to_string(), PathBuf::from("/usr/bin/g++"));
+        let config = {
+            let mut builder = WrapperConfig::new(address());
+            builder.add_executable("gcc".to_string(), PathBuf::from("/usr/bin/gcc"));
+            builder.add_executable("g++".to_string(), PathBuf::from("/usr/bin/g++"));
+            builder
+        };
 
         assert_eq!(config.get_executable("gcc"), Some(&PathBuf::from("/usr/bin/gcc")));
         assert_eq!(config.get_executable("g++"), Some(&PathBuf::from("/usr/bin/g++")));
@@ -284,30 +353,41 @@ mod tests {
 
     #[test]
     fn test_wrapper_config_reader_write_roundtrip() {
-        let mut original_config = WrapperConfig::new();
-        original_config.add_executable("gcc".to_string(), PathBuf::from("/usr/bin/gcc"));
-        original_config.add_executable("g++".to_string(), PathBuf::from("/usr/bin/g++"));
+        let original_config = {
+            let mut builder = WrapperConfig::new(address());
+            builder.add_executable("gcc".to_string(), PathBuf::from("/usr/bin/gcc"));
+            builder.add_executable("g++".to_string(), PathBuf::from("/usr/bin/g++"));
+            builder
+        };
 
         // Write to memory buffer
-        let mut buffer = Vec::new();
-        WrapperConfigWriter::write(&original_config, &mut buffer).unwrap();
+        let buffer = {
+            let mut writer = Vec::new();
+            WrapperConfigWriter::write(&original_config, &mut writer).unwrap();
+            writer
+        };
 
         // Read back from memory buffer
-        let cursor = Cursor::new(buffer);
-        let read_config = WrapperConfigReader::read(cursor).unwrap();
+        let read_config = {
+            let cursor = Cursor::new(buffer);
+            WrapperConfigReader::read(cursor).unwrap()
+        };
 
         assert_eq!(original_config, read_config);
     }
 
     #[test]
     fn test_wrapper_config_file_operations() {
+        let original_config = {
+            let mut builder = WrapperConfig::new(address());
+            builder.add_executable("gcc".to_string(), PathBuf::from("/usr/bin/gcc"));
+            builder.add_executable("clang".to_string(), PathBuf::from("/usr/bin/clang"));
+            builder
+        };
+
         let temp_dir = TempDir::new().unwrap();
         let config_path = temp_dir.path().join(CONFIG_FILENAME);
 
-        let mut original_config = WrapperConfig::new();
-        original_config.add_executable("gcc".to_string(), PathBuf::from("/usr/bin/gcc"));
-        original_config.add_executable("clang".to_string(), PathBuf::from("/usr/bin/clang"));
-
         // Write to file
         WrapperConfigWriter::write_to_file(&original_config, &config_path).unwrap();
         assert!(config_path.exists());
@@ -336,60 +416,126 @@ mod tests {
         assert!(matches!(result.unwrap_err(), ConfigError::Io(_)));
     }
 
+    #[test]
+    fn test_managed_directory_creation() {
+        let temp_dir = TempDir::new().unwrap();
+
+        let managed_dir = ManagedDirectory::create(temp_dir.path()).unwrap();
+
+        // Verify directory exists
+        assert!(managed_dir.path().exists());
+        assert!(managed_dir.path().ends_with(WRAPPER_DIR_NAME));
+    }
+
+    #[test]
+    fn test_managed_directory_cleanup_on_drop() {
+        let temp_dir = TempDir::new().unwrap();
+
+        let path;
+        {
+            let managed_dir = ManagedDirectory::create(temp_dir.path()).unwrap();
+            path = managed_dir.path().to_path_buf();
+            assert!(path.exists());
+        }
+        // After drop, directory should be cleaned up
+        assert!(!path.exists());
+    }
+
+    #[test]
+    fn test_managed_directory_replaces_existing() {
+        let temp_dir = TempDir::new().unwrap();
+
+        // Create first managed directory and add a file
+        let managed_dir1 = ManagedDirectory::create(temp_dir.path()).unwrap();
+        let test_file = managed_dir1.path().join("test_file.txt");
+        std::fs::write(&test_file, "test content").unwrap();
+        assert!(test_file.exists());
+
+        // Drop the first directory (but pretend we didn't clean up)
+        let path = managed_dir1.path().to_path_buf();
+        std::mem::forget(managed_dir1); // Simulate crash - don't run drop
+
+        // Verify the directory and file still exist (since we skipped drop)
+        assert!(path.exists());
+        assert!(test_file.exists());
+
+        // Create second managed directory in the SAME working directory - should replace existing
+        let managed_dir2 = ManagedDirectory::create(temp_dir.path()).unwrap();
+        assert_eq!(managed_dir2.path(), path);
+
+        // Old file should be gone (directory was replaced with a fresh one)
+        assert!(!test_file.exists());
+        // But the directory itself should exist (freshly created)
+        assert!(managed_dir2.path().exists());
+    }
+
     #[test]
     fn test_wrapper_directory_builder() {
         let temp_dir = TempDir::new().unwrap();
-        let wrapper_path = temp_dir.path().join("wrapper");
 
-        // Create wrapper executable
-        std::fs::write(&wrapper_path, "#!/bin/bash\necho wrapper").unwrap();
-
-        let mut builder = WrapperDirectoryBuilder::create(&wrapper_path, temp_dir.path()).unwrap();
-
-        // Register some executables
-        let gcc_wrapper = builder.register_executable(PathBuf::from("/usr/bin/gcc")).unwrap();
-        let gpp_wrapper = builder.register_executable(PathBuf::from("/usr/bin/g++")).unwrap();
+        let wrapper_dir = {
+            let wrapper_path = {
+                let file = temp_dir.path().join("wrapper");
+                std::fs::write(&file, "#!/bin/bash\necho wrapper").unwrap();
+                file
+            };
+            let mut builder =
+                WrapperDirectoryBuilder::create(&wrapper_path, temp_dir.path(), address()).unwrap();
+            builder.register_executable(PathBuf::from("/usr/bin/gcc")).unwrap();
+            builder.register_executable(PathBuf::from("/usr/bin/g++")).unwrap();
+            builder.build().unwrap()
+        };
 
         // Verify links were created
-        assert!(builder.path().join("gcc").exists());
-        assert!(builder.path().join("g++").exists());
-        assert_eq!(gcc_wrapper, builder.path().join("gcc"));
-        assert_eq!(gpp_wrapper, builder.path().join("g++"));
+        assert!(wrapper_dir.path().join("gcc").exists());
+        assert!(wrapper_dir.path().join("g++").exists());
+
+        // Verify wrapper directory is in .bear
+        assert!(wrapper_dir.path().ends_with(WRAPPER_DIR_NAME));
 
         // Finalize and check config file
-        let wrapper_dir = builder.build().unwrap();
-        assert!(wrapper_dir.path().join(CONFIG_FILENAME).exists());
+        let config_path = wrapper_dir.path().join(CONFIG_FILENAME);
+        assert!(config_path.exists(), "Config file should exist at {:?}", config_path);
+
         assert_eq!(wrapper_dir.config().executables.len(), 2);
         assert!(wrapper_dir.config().get_executable("gcc").is_some());
         assert!(wrapper_dir.config().get_executable("g++").is_some());
+
+        // Reading back the config file content, is matching with the config attribute
+        let wrapper_config = WrapperConfigReader::read_from_file(&config_path).unwrap();
+        assert_eq!(wrapper_config, wrapper_dir.config().clone());
     }
 
     #[test]
     fn test_wrapper_directory_duplicate_names() {
         let temp_dir = TempDir::new().unwrap();
-        let wrapper_path = temp_dir.path().join("wrapper");
+        let wrapper_path = {
+            let file = temp_dir.path().join("wrapper");
+            std::fs::write(&file, "#!/bin/bash\necho wrapper").unwrap();
+            file
+        };
 
-        std::fs::write(&wrapper_path, "#!/bin/bash\necho wrapper").unwrap();
-
-        let mut builder = WrapperDirectoryBuilder::create(&wrapper_path, temp_dir.path()).unwrap();
+        let mut builder = WrapperDirectoryBuilder::create(&wrapper_path, temp_dir.path(), address()).unwrap();
 
         // Register first executable
-        let _gcc_wrapper = builder.register_executable(PathBuf::from("/usr/bin/gcc")).unwrap();
+        let gcc_wrapper = builder.register_executable(PathBuf::from("/usr/bin/gcc")).unwrap();
 
         // Try to register different executable with same name - should fail
         let result = builder.register_executable(PathBuf::from("/usr/local/bin/gcc"));
         assert!(result.is_ok());
-        assert_eq!(result.unwrap(), _gcc_wrapper);
+        assert_eq!(result.unwrap(), gcc_wrapper);
     }
 
     #[test]
     fn test_wrapper_directory_same_executable_twice() {
         let temp_dir = TempDir::new().unwrap();
-        let wrapper_path = temp_dir.path().join("wrapper");
+        let wrapper_path = {
+            let file = temp_dir.path().join("wrapper");
+            std::fs::write(&file, "#!/bin/bash\necho wrapper").unwrap();
+            file
+        };
 
-        std::fs::write(&wrapper_path, "#!/bin/bash\necho wrapper").unwrap();
-
-        let mut builder = WrapperDirectoryBuilder::create(&wrapper_path, temp_dir.path()).unwrap();
+        let mut builder = WrapperDirectoryBuilder::create(&wrapper_path, temp_dir.path(), address()).unwrap();
 
         // Register same executable twice - should be OK
         let gcc_path = PathBuf::from("/usr/bin/gcc");
@@ -400,22 +546,4 @@ mod tests {
         let wrapper_dir = builder.build().unwrap();
         assert_eq!(wrapper_dir.config().executables.len(), 1); // Should only have one entry
     }
-
-    #[test]
-    fn test_wrapper_directory_temp_creation() {
-        let temp_parent = TempDir::new().unwrap();
-        let wrapper_path = temp_parent.path().join("wrapper");
-        std::fs::write(&wrapper_path, "#!/bin/bash\necho wrapper").unwrap();
-
-        let mut builder = WrapperDirectoryBuilder::create(&wrapper_path, temp_parent.path()).unwrap();
-
-        let clang_wrapper = builder.register_executable(PathBuf::from("/usr/bin/clang")).unwrap();
-
-        // Verify link was created in wrapper directory
-        assert!(builder.path().join("clang").exists());
-        assert_eq!(clang_wrapper, builder.path().join("clang"));
-
-        let wrapper_dir = builder.build().unwrap();
-        assert!(wrapper_dir.path().join(CONFIG_FILENAME).exists());
-    }
 }
