@@ -20,15 +20,15 @@
 //! should be generated:
 //!
 //! ## Cases that generate NO entries:
-//! 1. **Preprocessing-only commands**: Commands classified as `CompilerPass::Preprocessing`
-//! 2. **Info-only commands**: Commands classified as `CompilerPass::Info` (e.g., `--version`, `--help`)
+//! 1. **Preprocessing-only commands**: Commands with `PassEffect::StopsAt(Preprocessing)`
+//! 2. **Info-only commands**: Commands with `PassEffect::InfoAndExit` (e.g., `--version`, `--help`)
 //! 3. **Linking-only commands**: Commands without compilation flags and no compilable source files
 //! 4. **Commands without source files**: Any command that has no source files to process
 //!
 //! ## Cases that generate entries:
-//! 1. **Compilation commands**: Commands classified as `CompilerPass::Compiling` (compile-only or compile-to-assembly)
+//! 1. **Compilation commands**: Commands with `PassEffect::StopsAt(Compiling)` or `PassEffect::StopsAt(Assembling)`
 //! 2. **Compile-and-link commands**: Commands that both compile and link in one step
-//!    - Linking-specific flags (classified as `CompilerPass::Linking`) are filtered out from entries
+//!    - Linking-specific flags (classified as `PassEffect::Configures(Linking)`) are filtered out from entries
 //!    - Only compilation-relevant flags are included in the database
 //!
 //! The converter relies on semantic analysis performed by compiler interpreters to properly
@@ -41,16 +41,16 @@
 //! use bear::config::Format;
 //!
 //! let config = Format::default();
-//! let converter = CommandConverter::new(config).unwrap();
+//! let converter = CommandConverter::new(config);
 //!
 //! // The converter can be used to convert semantic Command instances
 //! // into compilation database entries based on the configured format
 //! ```
 
 use super::Entry;
-use super::{ConfigurablePathFormatter, FormatConfigurationError, PathFormatter};
+use super::{ConfigurablePathFormatter, PathFormatter};
 use crate::config;
-use crate::semantic::{ArgumentKind, Arguments, Command, CompilerCommand};
+use crate::semantic::{ArgumentKind, Arguments, Command, CompilerCommand, CompilerPass, PassEffect};
 use log::warn;
 use std::borrow::Cow;
 use std::path::{Path, PathBuf};
@@ -66,9 +66,9 @@ pub struct CommandConverter {
 
 impl CommandConverter {
     /// Creates a new CommandConverter with the specified format configuration.
-    pub fn new(format: config::Format) -> Result<Self, FormatConfigurationError> {
-        let path_formatter = Box::new(ConfigurablePathFormatter::new(format.paths)?);
-        Ok(Self { format: format.entries, path_formatter })
+    pub fn new(format: config::Format) -> Self {
+        let path_formatter = Box::new(ConfigurablePathFormatter::new(format.paths));
+        Self { format: format.entries, path_formatter }
     }
 
     /// Creates a new CommandConverter with a custom path formatter for testing.
@@ -100,8 +100,10 @@ impl CommandConverter {
         // Create output file if needed
         let output_file = self.create_output_file(&formatted_directory, &cmd.arguments);
 
-        // Create one entry per source argument
-        Self::find_arguments_by_kind(cmd, ArgumentKind::Source)
+        // Create one entry per source argument (only non-binary source files)
+        cmd.arguments
+            .iter()
+            .filter(|arg| matches!(arg.kind(), ArgumentKind::Source { binary: false }))
             .filter_map(|source_arg| {
                 // Get and format source file
                 let path_updater: &dyn Fn(&Path) -> Cow<Path> = &|path: &Path| Cow::Borrowed(path);
@@ -201,12 +203,16 @@ impl CommandConverter {
         // Add all non-source arguments, while handling source file placement
         for arg in &cmd.arguments {
             // Skip this specific source argument (using pointer equality)
-            if matches!(arg.kind(), ArgumentKind::Source) && !std::ptr::eq(arg.as_ref(), source_arg) {
+            if matches!(arg.kind(), ArgumentKind::Source { .. }) && !std::ptr::eq(arg.as_ref(), source_arg) {
                 continue;
             }
 
             // Filter out linking-specific arguments for compilation database entries
-            if matches!(arg.kind(), ArgumentKind::Other(Some(crate::semantic::CompilerPass::Linking))) {
+            if matches!(
+                arg.kind(),
+                ArgumentKind::Other(PassEffect::Configures(CompilerPass::Linking))
+                    | ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Linking))
+            ) {
                 continue;
             }
 
@@ -216,7 +222,7 @@ impl CommandConverter {
 
             // For file-type arguments, we need to format the paths
             match arg.kind() {
-                ArgumentKind::Source | ArgumentKind::Output => {
+                ArgumentKind::Source { .. } | ArgumentKind::Output => {
                     // These might contain file paths that need formatting
                     let formatted_args = original_args
                         .into_iter()
@@ -259,34 +265,42 @@ impl CommandConverter {
     /// Returns arguments of a specific kind from the command.
     ///
     /// This method filters arguments by their kind and returns their values as strings.
+    /// For `ArgumentKind::Source`, this matches any source regardless of the `binary` flag.
     fn find_arguments_by_kind(
         cmd: &CompilerCommand,
         kind: ArgumentKind,
     ) -> impl Iterator<Item = &Box<dyn Arguments>> {
-        cmd.arguments.iter().filter(move |arg| arg.kind() == kind)
+        cmd.arguments.iter().filter(move |arg| {
+            match (arg.kind(), kind) {
+                // For Source, match any source regardless of binary flag
+                (ArgumentKind::Source { .. }, ArgumentKind::Source { .. }) => true,
+                // For other kinds, use exact equality
+                (a, b) => a == b,
+            }
+        })
     }
 
     /// Determines if we should skip generating compilation database entries for a command.
     ///
     /// Returns true if the command should not generate entries for any of these reasons:
-    /// 1. Preprocessing-only commands (`CompilerPass::Preprocessing`)
-    /// 2. Info-only commands (`CompilerPass::Info`)
+    /// 1. Preprocessing-only commands (`PassEffect::StopsAt(Preprocessing)`)
+    /// 2. Info-only commands (`PassEffect::InfoAndExit`)
     /// 3. Commands without source files
     /// 4. Linking-only commands (no compilation flags and has source files)
     fn should_skip_entry_generation(&self, cmd: &CompilerCommand) -> bool {
-        // Check if this is a preprocessing-only command
-        if self.is_preprocessing_only(cmd) {
-            return true;
-        }
-
-        // Check if this is an info-only command
+        // Check if this is an info-only command (e.g., --version, --help)
         if self.is_info_only(cmd) {
             return true;
         }
 
-        // Find all source arguments
-        let source_arguments =
-            Self::find_arguments_by_kind(cmd, ArgumentKind::Source).collect::<Vec<&Box<dyn Arguments>>>();
+        // Check if this is a preprocessing-only command (e.g., -E)
+        if self.is_preprocessing_only(cmd) {
+            return true;
+        }
+
+        // Find all source arguments (using binary: false as a placeholder, find_arguments_by_kind matches any source)
+        let source_arguments = Self::find_arguments_by_kind(cmd, ArgumentKind::Source { binary: false })
+            .collect::<Vec<&Box<dyn Arguments>>>();
 
         // If no source files found, skip entry generation
         if source_arguments.is_empty() {
@@ -294,90 +308,73 @@ impl CommandConverter {
         }
 
         // Check if this is a linking-only command
-        self.is_linking_only(cmd)
+        if self.is_linking_only(cmd) {
+            return true;
+        }
+
+        false
     }
 
     /// Determines if a compiler command is preprocessing-only.
     ///
-    /// A command is considered preprocessing-only if it contains arguments
-    /// classified as `CompilerPass::Preprocessing` by the semantic analysis.
+    /// A command is considered preprocessing-only if it has a `PassEffect::StopsAt(Preprocessing)` flag.
+    /// This is the `-E` flag which explicitly stops the compiler after preprocessing.
     fn is_preprocessing_only(&self, cmd: &CompilerCommand) -> bool {
-        // A command is preprocessing-only if it has preprocessing flags but NO compilation flags
-        let has_preprocessing = cmd.arguments.iter().any(|arg| {
-            matches!(arg.kind(), ArgumentKind::Other(Some(crate::semantic::CompilerPass::Preprocessing)))
-        });
-
-        let has_compilation = cmd.arguments.iter().any(|arg| {
-            matches!(arg.kind(), ArgumentKind::Other(Some(crate::semantic::CompilerPass::Compiling)))
-        });
-
-        // Only preprocessing-only if it has preprocessing flags but no compilation flags
-        has_preprocessing && !has_compilation
+        cmd.arguments.iter().any(|arg| {
+            matches!(arg.kind(), ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Preprocessing)))
+        })
     }
 
     /// Determines if a compiler command is info-only.
     ///
     /// A command is considered info-only if it contains arguments
-    /// classified as `CompilerPass::Info` by the semantic analysis.
+    /// classified as `PassEffect::InfoAndExit` by the semantic analysis.
     /// These commands typically display information and don't perform compilation.
     fn is_info_only(&self, cmd: &CompilerCommand) -> bool {
-        cmd.arguments
-            .iter()
-            .any(|arg| matches!(arg.kind(), ArgumentKind::Other(Some(crate::semantic::CompilerPass::Info))))
+        cmd.arguments.iter().any(|arg| matches!(arg.kind(), ArgumentKind::Other(PassEffect::InfoAndExit)))
     }
 
     /// Determines if a compiler command is linking-only.
     ///
     /// A command is considered linking-only if:
-    /// 1. It does NOT have the `-c` flag (which means compile-only)
+    /// 1. It does NOT have a `PassEffect::StopsAt(Compiling)` or `PassEffect::StopsAt(Assembling)` flag
     /// 2. AND it has no compilable source files (only object files, libraries, etc.)
     ///
     /// This typically happens when linking pre-compiled object files or libraries.
     ///
-    /// Note: Arguments with `ArgumentKind::Source` have already been validated by
-    /// `looks_like_a_source_file()` during semantic analysis, but we still need to
-    /// exclude object files (.o, .a, etc.) which may be misclassified in tests.
+    /// The `binary` flag on `ArgumentKind::Source` is set during semantic analysis
+    /// by the interpreter, so we can simply check it here.
     fn is_linking_only(&self, cmd: &CompilerCommand) -> bool {
-        // Check if the command has compile-only flags
-        let has_compile_flag = cmd.arguments.iter().any(|arg| {
-            matches!(arg.kind(), ArgumentKind::Other(Some(crate::semantic::CompilerPass::Compiling)))
+        // Check if the command has a flag that stops before linking (-c or -S)
+        let stops_before_linking = cmd.arguments.iter().any(|arg| {
+            matches!(
+                arg.kind(),
+                ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Compiling))
+                    | ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Assembling))
+            )
         });
 
-        // If there's a -c flag, it's not linking-only
-        if has_compile_flag {
+        // If there's a -c or -S flag, it's not linking-only
+        if stops_before_linking {
             return false;
         }
 
-        // Check if there are any compilable source files (not object/library files)
+        // Check if there are any compilable source files (not binary files)
+        // The binary flag is set by the interpreter during semantic analysis
         let has_compilable_sources =
-            Self::find_arguments_by_kind(cmd, ArgumentKind::Source).any(|source_arg| {
-                let path_updater: &dyn Fn(&Path) -> Cow<Path> = &|path: &Path| Cow::Borrowed(path);
-                if let Some(file_path) = source_arg.as_file(path_updater) {
-                    // Check if this is a compilable source file (not an object file)
-                    if let Some(ext) = file_path.extension() {
-                        let ext_str = ext.to_string_lossy().to_lowercase();
-                        // Object files and libraries are not compilable source files
-                        !matches!(ext_str.as_str(), "o" | "a" | "so" | "dylib" | "dll" | "lib")
-                    } else {
-                        // Files without extensions are typically not object files
-                        true
-                    }
-                } else {
-                    false
-                }
-            });
+            cmd.arguments.iter().any(|arg| matches!(arg.kind(), ArgumentKind::Source { binary: false }));
 
-        // If no -c flag and no compilable sources, it's linking-only
+        // If no -c/-S flag and no compilable sources, it's linking-only
         !has_compilable_sources
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use super::super::super::clang::format::{FormatError, MockPathFormatter};
+    use super::super::format::{FormatError, MockPathFormatter};
     use super::*;
     use crate::config::{EntryFormat, Format, PathFormat};
-    use crate::semantic::{ArgumentKind, Command, CompilerCommand, CompilerPass};
+    use crate::semantic::{ArgumentKind, Command, CompilerCommand, CompilerPass, PassEffect};
     use std::io;
 
     #[test]
@@ -387,15 +384,15 @@ mod tests {
             "/usr/bin/gcc",
             vec![
                 (ArgumentKind::Compiler, vec!["/usr/bin/gcc"]),
-                (ArgumentKind::Other(Some(CompilerPass::Compiling)), vec!["-c"]),
-                (ArgumentKind::Other(None), vec!["-Wall"]),
-                (ArgumentKind::Source, vec!["main.c"]),
+                (ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Compiling)), vec!["-c"]),
+                (ArgumentKind::Other(PassEffect::None), vec!["-Wall"]),
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
                 (ArgumentKind::Output, vec!["-o", "main.o"]),
             ],
         ));
 
         let format = Format { paths: PathFormat::default(), entries: EntryFormat::default() };
-        let converter = CommandConverter::new(format).unwrap();
+        let converter = CommandConverter::new(format);
         let entries = converter.to_entries(&command);
 
         let expected = vec![Entry::from_arguments_str(
@@ -414,14 +411,14 @@ mod tests {
             "/usr/bin/g++",
             vec![
                 (ArgumentKind::Compiler, vec!["/usr/bin/g++"]),
-                (ArgumentKind::Other(Some(CompilerPass::Compiling)), vec!["-c"]),
-                (ArgumentKind::Source, vec!["file1.cpp"]),
-                (ArgumentKind::Source, vec!["file2.cpp"]),
+                (ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Compiling)), vec!["-c"]),
+                (ArgumentKind::Source { binary: false }, vec!["file1.cpp"]),
+                (ArgumentKind::Source { binary: false }, vec!["file2.cpp"]),
             ],
         ));
 
         let format = Format { paths: PathFormat::default(), entries: EntryFormat::default() };
-        let converter = CommandConverter::new(format).unwrap();
+        let converter = CommandConverter::new(format);
         let result = converter.to_entries(&command);
 
         let expected = vec![
@@ -436,11 +433,11 @@ mod tests {
         let command = Command::Compiler(CompilerCommand::from_strings(
             "/home/user",
             "gcc",
-            vec![(ArgumentKind::Other(Some(CompilerPass::Info)), vec!["--version"])],
+            vec![(ArgumentKind::Other(PassEffect::InfoAndExit), vec!["--version"])],
         ));
 
         let format = Format { paths: PathFormat::default(), entries: EntryFormat::default() };
-        let converter = CommandConverter::new(format).unwrap();
+        let converter = CommandConverter::new(format);
         let result = converter.to_entries(&command);
 
         let expected: Vec<Entry> = vec![];
@@ -454,8 +451,8 @@ mod tests {
             "/usr/bin/gcc",
             vec![
                 (ArgumentKind::Compiler, vec!["/usr/bin/gcc"]),
-                (ArgumentKind::Other(Some(CompilerPass::Compiling)), vec!["-c"]),
-                (ArgumentKind::Source, vec!["main.c"]),
+                (ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Compiling)), vec!["-c"]),
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
                 (ArgumentKind::Output, vec!["-o", "main.o"]),
             ],
         ));
@@ -463,7 +460,7 @@ mod tests {
             paths: PathFormat::default(),
             entries: EntryFormat { include_output_field: true, use_array_format: false },
         };
-        let converter = CommandConverter::new(format).unwrap();
+        let converter = CommandConverter::new(format);
         let entries = converter.to_entries(&command);
 
         let expected =
@@ -478,8 +475,8 @@ mod tests {
             "/usr/bin/gcc",
             vec![
                 (ArgumentKind::Compiler, vec!["/usr/bin/gcc"]),
-                (ArgumentKind::Other(Some(CompilerPass::Compiling)), vec!["-c"]),
-                (ArgumentKind::Source, vec!["main.c"]),
+                (ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Compiling)), vec!["-c"]),
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
                 (ArgumentKind::Output, vec!["-o", "main.o"]),
             ],
         ));
@@ -487,7 +484,7 @@ mod tests {
             paths: PathFormat::default(),
             entries: EntryFormat { use_array_format: true, include_output_field: false },
         };
-        let sut = CommandConverter::new(format).unwrap();
+        let sut = CommandConverter::new(format);
         let result = sut.to_entries(&command);
 
         let expected = vec![Entry::from_arguments_str(
@@ -506,14 +503,14 @@ mod tests {
             paths: PathFormat::default(),
             entries: EntryFormat { use_array_format: true, include_output_field: false },
         };
-        let converter = CommandConverter::new(format).unwrap();
+        let converter = CommandConverter::new(format);
 
         let compiler_cmd = CompilerCommand::from_strings(
             "/home/user",
             "/usr/bin/gcc",
             vec![
-                (ArgumentKind::Other(Some(CompilerPass::Compiling)), vec!["-c"]),
-                (ArgumentKind::Source, vec!["test.c"]),
+                (ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Compiling)), vec!["-c"]),
+                (ArgumentKind::Source { binary: false }, vec!["test.c"]),
             ],
         );
         let command = Command::Compiler(compiler_cmd);
@@ -543,7 +540,10 @@ mod tests {
         let compiler_cmd = CompilerCommand::from_strings(
             "/original/dir",
             "/usr/bin/gcc",
-            vec![(ArgumentKind::Source, vec!["main.c"]), (ArgumentKind::Output, vec!["-o", "main.o"])],
+            vec![
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
+                (ArgumentKind::Output, vec!["-o", "main.o"]),
+            ],
         );
         let command = Command::Compiler(compiler_cmd);
 
@@ -568,7 +568,7 @@ mod tests {
         let compiler_cmd = CompilerCommand::from_strings(
             "/nonexistent/dir",
             "/usr/bin/gcc",
-            vec![(ArgumentKind::Source, vec!["main.c"])],
+            vec![(ArgumentKind::Source { binary: false }, vec!["main.c"])],
         );
         let command = Command::Compiler(compiler_cmd);
 
@@ -594,7 +594,7 @@ mod tests {
         let compiler_cmd = CompilerCommand::from_strings(
             "/home/user",
             "/usr/bin/gcc",
-            vec![(ArgumentKind::Source, vec!["nonexistent.c"])],
+            vec![(ArgumentKind::Source { binary: false }, vec!["nonexistent.c"])],
         );
         let command = Command::Compiler(compiler_cmd);
 
@@ -637,7 +637,10 @@ mod tests {
         let compiler_cmd = CompilerCommand::from_strings(
             "/home/user",
             "/usr/bin/gcc",
-            vec![(ArgumentKind::Source, vec!["main.c"]), (ArgumentKind::Output, vec!["-o", "main.o"])],
+            vec![
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
+                (ArgumentKind::Output, vec!["-o", "main.o"]),
+            ],
         );
         let command = Command::Compiler(compiler_cmd);
 
@@ -649,30 +652,17 @@ mod tests {
         assert_eq!(entries[0].output, Some(PathBuf::from("main.o")));
     }
 
-    #[test]
-    fn test_configuration_validation_failure() {
-        use crate::config::{PathFormat, PathResolver};
-
-        let invalid_format = Format {
-            paths: PathFormat { directory: PathResolver::Relative, file: PathResolver::Absolute },
-            entries: EntryFormat::default(),
-        };
-
-        let result = CommandConverter::new(invalid_format);
-        assert!(result.is_err());
-    }
-
     #[test]
     fn test_preprocessing_only_command_no_entries() {
-        let format = Format { paths: PathFormat::default(), entries: EntryFormat::default() };
-        let converter = CommandConverter::new(format).unwrap();
+        let format = Format::default();
+        let converter = CommandConverter::new(format);
 
         let compiler_cmd = CompilerCommand::from_strings(
             "/home/user",
             "gcc",
             vec![
-                (ArgumentKind::Other(Some(CompilerPass::Preprocessing)), vec!["-E"]),
-                (ArgumentKind::Source, vec!["main.c"]),
+                (ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Preprocessing)), vec!["-E"]),
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
             ],
         );
         let command = Command::Compiler(compiler_cmd);
@@ -683,17 +673,18 @@ mod tests {
 
     #[test]
     fn test_linking_only_command_no_entries() {
-        let format = Format { paths: PathFormat::default(), entries: EntryFormat::default() };
-        let converter = CommandConverter::new(format).unwrap();
+        let format = Format::default();
+        let converter = CommandConverter::new(format);
 
         // Linking object files (no -c flag, object file inputs)
         let compiler_cmd = CompilerCommand::from_strings(
             "/home/user",
             "gcc",
             vec![
-                (ArgumentKind::Source, vec!["main.o", "lib.o"]),
+                (ArgumentKind::Source { binary: true }, vec!["main.o"]),
+                (ArgumentKind::Source { binary: true }, vec!["lib.o"]),
                 (ArgumentKind::Output, vec!["-o", "program"]),
-                (ArgumentKind::Other(Some(CompilerPass::Linking)), vec!["-L/usr/lib"]),
+                (ArgumentKind::Other(PassEffect::Configures(CompilerPass::Linking)), vec!["-L/usr/lib"]),
             ],
         );
         let command = Command::Compiler(compiler_cmd);
@@ -704,15 +695,15 @@ mod tests {
 
     #[test]
     fn test_compile_only_command_generates_entries() {
-        let format = Format { paths: PathFormat::default(), entries: EntryFormat::default() };
-        let converter = CommandConverter::new(format).unwrap();
+        let format = Format::default();
+        let converter = CommandConverter::new(format);
 
         let compiler_cmd = CompilerCommand::from_strings(
             "/home/user",
             "gcc",
             vec![
-                (ArgumentKind::Other(Some(CompilerPass::Compiling)), vec!["-c"]),
-                (ArgumentKind::Source, vec!["main.c"]),
+                (ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Compiling)), vec!["-c"]),
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
                 (ArgumentKind::Output, vec!["-o", "main.o"]),
             ],
         );
@@ -729,16 +720,16 @@ mod tests {
             paths: PathFormat::default(),
             entries: EntryFormat { use_array_format: true, include_output_field: false },
         };
-        let converter = CommandConverter::new(format).unwrap();
+        let converter = CommandConverter::new(format);
 
         let compiler_cmd = CompilerCommand::from_strings(
             "/home/user",
             "gcc",
             vec![
-                (ArgumentKind::Source, vec!["main.c"]),
-                (ArgumentKind::Other(Some(CompilerPass::Linking)), vec!["-L/usr/lib"]),
-                (ArgumentKind::Other(Some(CompilerPass::Linking)), vec!["-lmath"]),
-                (ArgumentKind::Other(Some(CompilerPass::Compiling)), vec!["-Wall"]),
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
+                (ArgumentKind::Other(PassEffect::Configures(CompilerPass::Linking)), vec!["-L/usr/lib"]),
+                (ArgumentKind::Other(PassEffect::Configures(CompilerPass::Linking)), vec!["-lmath"]),
+                (ArgumentKind::Other(PassEffect::Configures(CompilerPass::Compiling)), vec!["-Wall"]),
             ],
         );
         let command = Command::Compiler(compiler_cmd);
@@ -758,13 +749,13 @@ mod tests {
 
     #[test]
     fn test_info_command_no_entries() {
-        let format = Format { paths: PathFormat::default(), entries: EntryFormat::default() };
-        let converter = CommandConverter::new(format).unwrap();
+        let format = Format::default();
+        let converter = CommandConverter::new(format);
 
         let compiler_cmd = CompilerCommand::from_strings(
             "/home/user",
             "gcc",
-            vec![(ArgumentKind::Other(Some(CompilerPass::Info)), vec!["--version"])],
+            vec![(ArgumentKind::Other(PassEffect::InfoAndExit), vec!["--version"])],
         );
         let command = Command::Compiler(compiler_cmd);
 
@@ -774,17 +765,17 @@ mod tests {
 
     #[test]
     fn test_realistic_source_file_detection() {
-        let format = Format { paths: PathFormat::default(), entries: EntryFormat::default() };
-        let converter = CommandConverter::new(format).unwrap();
+        let format = Format::default();
+        let converter = CommandConverter::new(format);
 
         // Test compile-and-link with real source files (should generate entries)
         let compiler_cmd = CompilerCommand::from_strings(
             "/home/user",
             "gcc",
             vec![
-                (ArgumentKind::Source, vec!["main.c"]),    // Real source file
-                (ArgumentKind::Source, vec!["utils.cpp"]), // Real source file
-                (ArgumentKind::Other(Some(CompilerPass::Linking)), vec!["-lm"]),
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]), // Real source file
+                (ArgumentKind::Source { binary: false }, vec!["utils.cpp"]), // Real source file
+                (ArgumentKind::Other(PassEffect::Configures(CompilerPass::Linking)), vec!["-lm"]),
             ],
         );
         let command = Command::Compiler(compiler_cmd);
@@ -797,8 +788,8 @@ mod tests {
             "/home/user",
             "gcc",
             vec![
-                (ArgumentKind::Source, vec!["main.o"]),  // Object file
-                (ArgumentKind::Source, vec!["utils.a"]), // Static library
+                (ArgumentKind::Source { binary: true }, vec!["main.o"]), // Object file
+                (ArgumentKind::Source { binary: true }, vec!["utils.a"]), // Static library
                 (ArgumentKind::Output, vec!["-o", "program"]),
             ],
         );
@@ -810,8 +801,8 @@ mod tests {
 
     #[test]
     fn test_semantic_classification_vs_raw_flags() {
-        let format = Format { paths: PathFormat::default(), entries: EntryFormat::default() };
-        let converter = CommandConverter::new(format).unwrap();
+        let format = Format::default();
+        let converter = CommandConverter::new(format);
 
         // Test that we rely on semantic classification, not raw flag strings
         // This tests a hypothetical case where a flag might look like "-E" but
@@ -823,10 +814,10 @@ mod tests {
             "gcc",
             vec![
                 (
-                    ArgumentKind::Other(Some(CompilerPass::Preprocessing)),
+                    ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Preprocessing)),
                     vec!["-E"], // Semantically classified as preprocessing
                 ),
-                (ArgumentKind::Source, vec!["main.c"]),
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
             ],
         );
         let command = Command::Compiler(compiler_cmd);
@@ -839,10 +830,10 @@ mod tests {
             "gcc",
             vec![
                 (
-                    ArgumentKind::Other(Some(CompilerPass::Compiling)),
+                    ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Compiling)),
                     vec!["-c"], // Semantically classified as compiling
                 ),
-                (ArgumentKind::Source, vec!["main.c"]),
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
             ],
         );
         let command = Command::Compiler(compiler_cmd);
@@ -854,7 +845,7 @@ mod tests {
             "/home/user",
             "gcc",
             vec![(
-                ArgumentKind::Other(Some(CompilerPass::Info)),
+                ArgumentKind::Other(PassEffect::InfoAndExit),
                 vec!["--version"], // Semantically classified as info
             )],
         );
@@ -867,13 +858,13 @@ mod tests {
             "/home/user",
             "gcc",
             vec![
-                (ArgumentKind::Source, vec!["main.c"]),
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
                 (
-                    ArgumentKind::Other(Some(CompilerPass::Linking)),
+                    ArgumentKind::Other(PassEffect::Configures(CompilerPass::Linking)),
                     vec!["-lmath"], // Semantically classified as linking
                 ),
                 (
-                    ArgumentKind::Other(Some(CompilerPass::Compiling)),
+                    ArgumentKind::Other(PassEffect::Configures(CompilerPass::Compiling)),
                     vec!["-O2"], // Compilation flag
                 ),
             ],
@@ -894,7 +885,7 @@ mod tests {
             paths: PathFormat::default(),
             entries: EntryFormat { use_array_format: true, include_output_field: true },
         };
-        let converter = CommandConverter::new(format).unwrap();
+        let converter = CommandConverter::new(format);
 
         // Test that all three formatting methods work consistently
         let compiler_cmd = CompilerCommand::from_strings(
@@ -902,8 +893,8 @@ mod tests {
             "gcc",
             vec![
                 (ArgumentKind::Compiler, vec!["gcc"]),
-                (ArgumentKind::Other(Some(CompilerPass::Compiling)), vec!["-c"]),
-                (ArgumentKind::Source, vec!["main.c"]),
+                (ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Compiling)), vec!["-c"]),
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
                 (ArgumentKind::Output, vec!["-o", "main.o"]),
             ],
         );
@@ -934,8 +925,8 @@ mod tests {
 
     #[test]
     fn test_preprocessing_and_compilation_flags_generates_entries() {
-        let format = Format { paths: PathFormat::default(), entries: EntryFormat::default() };
-        let converter = CommandConverter::new(format).unwrap();
+        let format = Format::default();
+        let converter = CommandConverter::new(format);
 
         // Test command with both preprocessing flags (-D) and compilation flags (-c)
         let compiler_cmd = CompilerCommand::from_strings(
@@ -943,9 +934,12 @@ mod tests {
             "gcc",
             vec![
                 (ArgumentKind::Compiler, vec!["gcc"]),
-                (ArgumentKind::Other(Some(CompilerPass::Preprocessing)), vec!["-DWRAPPER_FLAG"]),
-                (ArgumentKind::Other(Some(CompilerPass::Compiling)), vec!["-c"]),
-                (ArgumentKind::Source, vec!["test.c"]),
+                (
+                    ArgumentKind::Other(PassEffect::Configures(CompilerPass::Preprocessing)),
+                    vec!["-DWRAPPER_FLAG"],
+                ),
+                (ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Compiling)), vec!["-c"]),
+                (ArgumentKind::Source { binary: false }, vec!["test.c"]),
             ],
         );
         let command = Command::Compiler(compiler_cmd);
@@ -968,8 +962,8 @@ mod tests {
 
     #[test]
     fn test_preprocessing_only_with_defines_no_entries() {
-        let format = Format { paths: PathFormat::default(), entries: EntryFormat::default() };
-        let converter = CommandConverter::new(format).unwrap();
+        let format = Format::default();
+        let converter = CommandConverter::new(format);
 
         // Test command with only preprocessing flags (no -c flag)
         let compiler_cmd = CompilerCommand::from_strings(
@@ -977,16 +971,47 @@ mod tests {
             "gcc",
             vec![
                 (ArgumentKind::Compiler, vec!["gcc"]),
-                (ArgumentKind::Other(Some(CompilerPass::Preprocessing)), vec!["-E"]),
-                (ArgumentKind::Other(Some(CompilerPass::Preprocessing)), vec!["-DSOME_DEFINE"]),
-                (ArgumentKind::Source, vec!["test.c"]),
+                (ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Preprocessing)), vec!["-E"]),
+                (
+                    ArgumentKind::Other(PassEffect::Configures(CompilerPass::Preprocessing)),
+                    vec!["-DSOME_DEFINE"],
+                ),
+                (ArgumentKind::Source { binary: false }, vec!["test.c"]),
             ],
         );
         let command = Command::Compiler(compiler_cmd);
 
         let result = converter.to_entries(&command);
 
-        // Should NOT generate entries because it's preprocessing-only (no compilation flags)
+        // Should NOT generate entries because it's preprocessing-only (has -E flag)
         assert_eq!(result.len(), 0);
     }
+
+    #[test]
+    fn test_driver_option_does_not_affect_entry_generation() {
+        let format = Format::default();
+        let converter = CommandConverter::new(format);
+
+        // Test command with driver options like -pipe (should still generate entries)
+        let compiler_cmd = CompilerCommand::from_strings(
+            "/home/user",
+            "gcc",
+            vec![
+                (ArgumentKind::Compiler, vec!["gcc"]),
+                (ArgumentKind::Other(PassEffect::DriverOption), vec!["-pipe"]),
+                (ArgumentKind::Other(PassEffect::StopsAt(CompilerPass::Compiling)), vec!["-c"]),
+                (ArgumentKind::Source { binary: false }, vec!["main.c"]),
+            ],
+        );
+        let command = Command::Compiler(compiler_cmd);
+
+        let result = converter.to_entries(&command);
+
+        // Should generate entries - driver options don't stop compilation
+        assert_eq!(result.len(), 1);
+        assert_eq!(result[0].file, PathBuf::from("main.c"));
+
+        // Verify -pipe is included in the command
+        assert!(result[0].arguments.contains(&"-pipe".to_string()));
+    }
 }
