@@ -42,7 +42,6 @@
 use super::constants::*;
 use anyhow::{Context, Result};
 use assert_cmd::Command;
-use assert_cmd::cargo::cargo_bin;
 // TempDir and predicates are infrastructure for future tests
 #[allow(unused_imports)]
 use assert_fs::{TempDir, prelude::*};
@@ -199,6 +198,7 @@ impl TestEnvironment {
     }
 
     /// Verify that a file is encoded with the specified encoding
+    #[allow(dead_code)]
     pub fn verify_file_encoding(
         &self,
         file_path: &Path,
@@ -231,7 +231,7 @@ impl TestEnvironment {
 
     /// Run bear with the given arguments
     pub fn run_bear(&self, args: &[&str]) -> Result<BearOutput> {
-        let mut cmd = Command::new(cargo_bin(BEAR_BIN));
+        let mut cmd = Command::new(BEAR_EXECUTABLE_PATH);
         cmd.current_dir(self.temp_dir()).env("RUST_LOG", "debug").env("RUST_BACKTRACE", "1").args(args);
 
         let output = cmd.output()?;
@@ -352,7 +352,7 @@ impl TestEnvironment {
 
         let events: Vec<Value> = content
             .lines()
-            .map(|line| serde_json::from_str(line))
+            .map(serde_json::from_str)
             .collect::<Result<Vec<_>, _>>()
             .with_context(|| "Failed to parse events JSON lines")?;
 
@@ -890,10 +890,8 @@ impl EventMatcher {
 
     fn matches(&self, event: &Value) -> bool {
         // Check event type if specified
-        if let Some(ref expected_type) = self.event_type {
-            if !event.get(expected_type).is_some() {
-                return false;
-            }
+        if self.event_type.as_ref().is_some_and(|expected_type| event.get(expected_type).is_none()) {
+            return false;
         }
 
         // Get the execution part of the event (most common case)
