@@ -1,127 +1,48 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
-//! This file implements a shared library. This library can be pre-loaded by
-//! the dynamic linker of the Operating System (OS). It implements a few function
-//! related to process creation. By pre-load this library the executed process
-//! uses these functions instead of those from the standard library.
+//! This file implements the Rust side of the preload library interception.
 //!
-//! The idea here is to inject a logic before calling the real methods. The logic is
-//! to dump the call into a file. To call the real method, this library is doing
-//! the job of the dynamic linker.
+//! All exported functions are prefixed with `rust_` and are called from the C shim
+//! (`src/c/shim.c`). This separation exists because:
 //!
-//! The only input for the log writing is about the destination directory.
-//! This is passed as an environment variable.
+//! 1. Stable Rust cannot handle C variadic arguments (execl family)
+//! 2. On FreeBSD, libc functions may call each other internally. By having all
+//!    exported symbols in C call into Rust (which uses dlsym(RTLD_NEXT, ...)),
+//!    we avoid recursive interception issues.
+//!
+//! Each `rust_*` function:
+//! - Reports the execution to the collector
+//! - Optionally "doctors" the environment to ensure child processes continue interception
+//! - Calls the real function via dlsym(RTLD_NEXT, ...)
+//!
+//! ## Environment Doctoring
+//!
+//! When SESSION is set (meaning we're in an interception session), we check if the
+//! environment passed to exec functions still has the correct preload settings.
+//! If not, we "doctor" the environment to restore them, ensuring child processes
+//! continue to be intercepted even if the build system cleared the variables.
 
 use std::collections::HashMap;
-use std::ffi::{CStr, CString};
-#[cfg(not(test))]
-use std::io::Write;
+use std::ffi::CStr;
 use std::path::PathBuf;
 use std::ptr;
+use std::sync::OnceLock;
 use std::sync::atomic::{AtomicPtr, Ordering};
 
 use bear::intercept::reporter::{Reporter, ReporterFactory};
 use bear::intercept::tcp::ReporterOnTcp;
 use bear::intercept::{Event, Execution};
 use ctor::ctor;
-use libc::{c_char, c_int, pid_t, posix_spawn_file_actions_t, posix_spawnattr_t};
+use libc::{RTLD_NEXT, c_char, c_int, pid_t, posix_spawn_file_actions_t, posix_spawnattr_t};
 
-// Function pointer types for the original functions
-#[cfg(has_symbol_execve)]
-type ExecveFunc = unsafe extern "C" fn(
-    path: *const c_char,
-    argv: *const *const c_char,
-    envp: *const *const c_char,
-) -> c_int;
+use crate::session::{DoctoredEnvironment, SESSION, in_session, init_session_from_envp};
 
-#[cfg(has_symbol_execv)]
-type ExecvFunc = unsafe extern "C" fn(path: *const c_char, argv: *const *const c_char) -> c_int;
-
-#[cfg(has_symbol_execvpe)]
-type ExecvpeFunc = unsafe extern "C" fn(
-    file: *const c_char,
-    argv: *const *const c_char,
-    envp: *const *const c_char,
-) -> c_int;
-
-#[cfg(has_symbol_execvp)]
-type ExecvpFunc = unsafe extern "C" fn(file: *const c_char, argv: *const *const c_char) -> c_int;
-
-#[cfg(has_symbol_execvP)]
-type ExecvPFunc = unsafe extern "C" fn(
-    file: *const c_char,
-    search_path: *const c_char,
-    argv: *const *const c_char,
-) -> c_int;
-
-#[cfg(has_symbol_exect)]
-type ExectFunc = unsafe extern "C" fn(
-    path: *const c_char,
-    argv: *const *const c_char,
-    envp: *const *const c_char,
-) -> c_int;
-
-#[cfg(has_symbol_posix_spawn)]
-type PosixSpawnFunc = unsafe extern "C" fn(
-    pid: *mut pid_t,
-    path: *const c_char,
-    file_actions: *const posix_spawn_file_actions_t,
-    attrp: *const posix_spawnattr_t,
-    argv: *const *const c_char,
-    envp: *const *const c_char,
-) -> c_int;
-
-#[cfg(has_symbol_posix_spawnp)]
-type PosixSpawnpFunc = unsafe extern "C" fn(
-    pid: *mut pid_t,
-    file: *const c_char,
-    file_actions: *const posix_spawn_file_actions_t,
-    attrp: *const posix_spawnattr_t,
-    argv: *const *const c_char,
-    envp: *const *const c_char,
-) -> c_int;
-
-#[cfg(has_symbol_popen)]
-type PopenFunc = unsafe extern "C" fn(command: *const c_char, mode: *const c_char) -> *mut libc::FILE;
-
-#[cfg(has_symbol_system)]
-type SystemFunc = unsafe extern "C" fn(command: *const c_char) -> c_int;
-
-// Dynamic loading related constants and types
-const RTLD_NEXT: *mut libc::c_void = -1isize as *mut libc::c_void;
-
-/// Constructor function that is called when the library is loaded
-///
-/// # Safety
-/// This function is unsafe because it modifies global state.
-#[ctor]
-unsafe fn on_load() {
-    #[cfg(not(test))]
-    {
-        let pid = std::process::id();
-        env_logger::Builder::from_default_env()
-            .format(move |buf, record| {
-                let timestamp = buf.timestamp();
-                writeln!(buf, "[{timestamp} preload/{pid}] {}", record.args())
-            })
-            .init();
-    }
-    log::debug!("Initializing intercept-preload library");
-}
-
-// Static variables to hold original function pointers
 #[ctor]
 static REAL_EXECVE: AtomicPtr<libc::c_void> = {
     let ptr = unsafe { libc::dlsym(RTLD_NEXT, c"execve".as_ptr() as *const _) };
     AtomicPtr::new(ptr)
 };
 
-#[ctor]
-static REAL_EXECV: AtomicPtr<libc::c_void> = {
-    let ptr = unsafe { libc::dlsym(RTLD_NEXT, c"execv".as_ptr() as *const _) };
-    AtomicPtr::new(ptr)
-};
-
 #[ctor]
 static REAL_EXECVPE: AtomicPtr<libc::c_void> = {
     let ptr = unsafe { libc::dlsym(RTLD_NEXT, c"execvpe".as_ptr() as *const _) };
@@ -135,7 +56,7 @@ static REAL_EXECVP: AtomicPtr<libc::c_void> = {
 };
 
 #[ctor]
-static REAL_EXECVE_OPENBSD: AtomicPtr<libc::c_void> = {
+static REAL_EXECVP_OPENBSD: AtomicPtr<libc::c_void> = {
     let ptr = unsafe { libc::dlsym(RTLD_NEXT, c"execvP".as_ptr() as *const _) };
     AtomicPtr::new(ptr)
 };
@@ -158,36 +79,84 @@ static REAL_POSIX_SPAWNP: AtomicPtr<libc::c_void> = {
     AtomicPtr::new(ptr)
 };
 
-#[cfg(has_symbol_popen)]
 #[ctor]
 static REAL_POPEN: AtomicPtr<libc::c_void> = {
     let ptr = unsafe { libc::dlsym(RTLD_NEXT, c"popen".as_ptr() as *const _) };
     AtomicPtr::new(ptr)
 };
 
-#[cfg(has_symbol_system)]
 #[ctor]
 static REAL_SYSTEM: AtomicPtr<libc::c_void> = {
     let ptr = unsafe { libc::dlsym(RTLD_NEXT, c"system".as_ptr() as *const _) };
     AtomicPtr::new(ptr)
 };
 
-#[ctor]
-static REPORTER: AtomicPtr<ReporterOnTcp> = {
-    // This will capture the reporter address for later use, before any environment variable
-    // might be changed.
-    ReporterFactory::create_as_ptr()
-};
+/// Reporter for sending execution events to the collector.
+/// Initialized once by `rust_session_init` when called from the C shim constructor.
+/// Using `OnceLock` ensures safe single-initialization without raw pointers and
+/// provides `&ReporterOnTcp` references that are sound across threads (since
+/// `ReporterOnTcp` is `Sync` — it only holds a `SocketAddr` and creates a fresh
+/// `TcpStream` per report call).
+static REPORTER: OnceLock<ReporterOnTcp> = OnceLock::new();
 
+/// Initialize the session from the environment.
+///
+/// Called from the C shim's `on_load()` constructor. This captures the
+/// environment variables before the build system has a chance to clear them.
+///
 /// # Safety
-/// This function is unsafe because it modifies global state.
+/// This function is unsafe because it dereferences raw pointers from C.
+#[unsafe(no_mangle)]
+pub unsafe extern "C" fn rust_session_init(envp: *const *const c_char) {
+    // Initialize logging first
+    #[cfg(not(test))]
+    {
+        use std::io::Write;
+
+        let pid = std::process::id();
+        let _ = env_logger::Builder::from_default_env()
+            .format(move |buf, record| {
+                let timestamp = buf.timestamp();
+                writeln!(buf, "[{timestamp} preload/{pid}] {}", record.args())
+            })
+            .try_init();
+    }
+
+    // Initialize the session and get the destination for reporter setup
+    let destination = unsafe { init_session_from_envp(envp) };
+
+    // Initialize the reporter from the captured destination
+    if let Some(address) = destination {
+        let reporter = ReporterFactory::create(address);
+        if REPORTER.set(reporter).is_err() {
+            log::warn!("Reporter already initialized, ignoring duplicate init");
+        }
+    } else {
+        log::debug!("No destination found, reporter not initialized");
+    }
+}
+
+/// Rust implementation of execve
+///
+/// Called from C shim for: execv, execve, execl, execle
+///
+/// # Safety
+/// This function is unsafe because it:
+/// - Dereferences raw pointers
+/// - Calls the real execve function
 #[cfg(has_symbol_execve)]
 #[unsafe(no_mangle)]
-pub unsafe extern "C" fn execve(
+pub unsafe extern "C" fn rust_execve(
     path: *const c_char,
     argv: *const *const c_char,
     envp: *const *const c_char,
 ) -> c_int {
+    type ExecveFunc = unsafe extern "C" fn(
+        path: *const c_char,
+        argv: *const *const c_char,
+        envp: *const *const c_char,
+    ) -> c_int;
+
     unsafe {
         report(|| {
             let result = Execution {
@@ -199,55 +168,44 @@ pub unsafe extern "C" fn execve(
             Ok(result)
         });
 
+        // Doctor the environment if session is active but the build system stripped preload vars
+        let resolved_env = resolve_environment(envp);
+        let envp_to_use = resolved_env.as_ptr();
+
         let func_ptr = REAL_EXECVE.load(Ordering::SeqCst);
         if !func_ptr.is_null() {
             let real_func_ptr: ExecveFunc = std::mem::transmute(func_ptr);
-            real_func_ptr(path, argv, envp)
+            real_func_ptr(path, argv, envp_to_use)
         } else {
             log::error!("Real execve function not found");
-            libc::ENOSYS
+            set_errno(libc::ENOSYS);
+            -1
         }
     }
 }
 
+/// Rust implementation of execvpe (GNU extension)
+///
+/// Called from C shim for: execvpe
+/// On platforms where execvpe is available, execlp and execvp are also routed here.
+///
 /// # Safety
-/// This function is unsafe because it modifies global state.
-#[cfg(has_symbol_execv)]
-#[unsafe(no_mangle)]
-pub unsafe extern "C" fn execv(path: *const c_char, argv: *const *const c_char) -> c_int {
-    // Try to report but don't fail if we can't
-    unsafe {
-        report(|| {
-            let result = Execution {
-                executable: as_path_buf(path)?,
-                arguments: as_string_vec(argv)?,
-                working_dir: working_dir()?,
-                environment: std::env::vars().collect(),
-            };
-            Ok(result)
-        });
-
-        let func_ptr = REAL_EXECV.load(Ordering::SeqCst);
-        if !func_ptr.is_null() {
-            let real_func_ptr: ExecvFunc = std::mem::transmute(func_ptr);
-            real_func_ptr(path, argv)
-        } else {
-            log::error!("Real execv function not found");
-            libc::ENOSYS
-        }
-    }
-}
-
-/// # Safety
-/// This function is unsafe because it modifies global state.
+/// This function is unsafe because it:
+/// - Dereferences raw pointers
+/// - Calls the real execvpe function
 #[cfg(has_symbol_execvpe)]
 #[unsafe(no_mangle)]
-pub unsafe extern "C" fn execvpe(
+pub unsafe extern "C" fn rust_execvpe(
     file: *const c_char,
     argv: *const *const c_char,
     envp: *const *const c_char,
 ) -> c_int {
-    // Try to report but don't fail if we can't
+    type ExecvpeFunc = unsafe extern "C" fn(
+        file: *const c_char,
+        argv: *const *const c_char,
+        envp: *const *const c_char,
+    ) -> c_int;
+
     unsafe {
         report(|| {
             let result = Execution {
@@ -259,23 +217,39 @@ pub unsafe extern "C" fn execvpe(
             Ok(result)
         });
 
+        // Doctor the environment if session is active but the build system stripped preload vars
+        let resolved_env = resolve_environment(envp);
+        let envp_to_use = resolved_env.as_ptr();
+
         let func_ptr = REAL_EXECVPE.load(Ordering::SeqCst);
         if !func_ptr.is_null() {
             let real_func_ptr: ExecvpeFunc = std::mem::transmute(func_ptr);
-            real_func_ptr(file, argv, envp)
+            real_func_ptr(file, argv, envp_to_use)
         } else {
             log::error!("Real execvpe function not found");
-            libc::ENOSYS
+            set_errno(libc::ENOSYS);
+            -1
         }
     }
 }
 
+/// Rust implementation of execvp
+///
+/// Called from C shim for: execvp, execlp (on platforms where execvpe is not available)
+///
+/// Unlike `rust_execvpe`, this function does not take an explicit `envp` argument.
+/// The real `execvp` uses the process's `environ`, so no environment doctoring is
+/// performed here (same approach as `popen` and `system`).
+///
 /// # Safety
-/// This function is unsafe because it modifies global state.
+/// This function is unsafe because it:
+/// - Dereferences raw pointers
+/// - Calls the real execvp function
 #[cfg(has_symbol_execvp)]
 #[unsafe(no_mangle)]
-pub unsafe extern "C" fn execvp(file: *const c_char, argv: *const *const c_char) -> c_int {
-    // Try to report but don't fail if we can't
+pub unsafe extern "C" fn rust_execvp(file: *const c_char, argv: *const *const c_char) -> c_int {
+    type ExecvpFunc = unsafe extern "C" fn(file: *const c_char, argv: *const *const c_char) -> c_int;
+
     unsafe {
         report(|| {
             let result = Execution {
@@ -293,20 +267,33 @@ pub unsafe extern "C" fn execvp(file: *const c_char, argv: *const *const c_char)
             real_func_ptr(file, argv)
         } else {
             log::error!("Real execvp function not found");
-            libc::ENOSYS
+            set_errno(libc::ENOSYS);
+            -1
         }
     }
 }
 
+/// Rust implementation of execvP (BSD extension)
+///
+/// Called from C shim for: execvP
+///
 /// # Safety
-/// This function is unsafe because it modifies global state.
+/// This function is unsafe because it:
+/// - Dereferences raw pointers
+/// - Calls the real execvP function
 #[cfg(has_symbol_execvP)]
 #[unsafe(no_mangle)]
-pub unsafe extern "C" fn execvP(
+pub unsafe extern "C" fn rust_execvP(
     file: *const c_char,
     search_path: *const c_char,
     argv: *const *const c_char,
 ) -> c_int {
+    type ExecvPFunc = unsafe extern "C" fn(
+        file: *const c_char,
+        search_path: *const c_char,
+        argv: *const *const c_char,
+    ) -> c_int;
+
     unsafe {
         report(|| {
             let result = Execution {
@@ -318,26 +305,39 @@ pub unsafe extern "C" fn execvP(
             Ok(result)
         });
 
-        let func_ptr = REAL_EXECVE_OPENBSD.load(Ordering::SeqCst);
+        let func_ptr = REAL_EXECVP_OPENBSD.load(Ordering::SeqCst);
         if !func_ptr.is_null() {
             let real_func_ptr: ExecvPFunc = std::mem::transmute(func_ptr);
             real_func_ptr(file, search_path, argv)
         } else {
             log::error!("Real execvP function not found");
-            libc::ENOSYS
+            set_errno(libc::ENOSYS);
+            -1
         }
     }
 }
 
+/// Rust implementation of exect (BSD, deprecated)
+///
+/// Called from C shim for: exect
+///
 /// # Safety
-/// This function is unsafe because it modifies global state.
+/// This function is unsafe because it:
+/// - Dereferences raw pointers
+/// - Calls the real exect function
 #[cfg(has_symbol_exect)]
 #[unsafe(no_mangle)]
-pub unsafe extern "C" fn exect(
+pub unsafe extern "C" fn rust_exect(
     path: *const c_char,
     argv: *const *const c_char,
     envp: *const *const c_char,
 ) -> c_int {
+    type ExectFunc = unsafe extern "C" fn(
+        path: *const c_char,
+        argv: *const *const c_char,
+        envp: *const *const c_char,
+    ) -> c_int;
+
     unsafe {
         report(|| {
             let result = Execution {
@@ -349,135 +349,33 @@ pub unsafe extern "C" fn exect(
             Ok(result)
         });
 
+        // Doctor the environment if session is active but the build system stripped preload vars
+        let resolved_env = resolve_environment(envp);
+        let envp_to_use = resolved_env.as_ptr();
+
         let func_ptr = REAL_EXECT.load(Ordering::SeqCst);
         if !func_ptr.is_null() {
             let real_func_ptr: ExectFunc = std::mem::transmute(func_ptr);
-            real_func_ptr(path, argv, envp)
+            real_func_ptr(path, argv, envp_to_use)
         } else {
             log::error!("Real exect function not found");
-            libc::ENOSYS
+            set_errno(libc::ENOSYS);
+            -1
         }
     }
 }
 
-// Implementations for variable argument functions
-// We need to handle C variadic arguments in Rust
-
+/// Rust implementation of posix_spawn
+///
+/// Called from C shim for: posix_spawn
+///
 /// # Safety
-/// This function is unsafe because it modifies global state.
-#[cfg(all(has_symbol_execl, has_symbol_execv))]
-#[unsafe(no_mangle)]
-pub unsafe extern "C" fn execl(
-    path: *const c_char,
-    arg: *const c_char,
-    args: *const c_char, /* variadic */
-) -> c_int {
-    // In a real implementation, we would need to pass all the variadic arguments
-    // But since we can't access all of them directly in Rust, we'll collect what we can
-    // and use execv instead
-
-    // Create a vector of argument pointers that we can access
-    let mut argv = Vec::new();
-    argv.push(path);
-
-    if !arg.is_null() {
-        argv.push(arg);
-
-        // In C, execl() is often implemented with execv() and just copying the
-        // arguments we have access to
-        let va_arg = args;
-        if !va_arg.is_null() {
-            argv.push(va_arg);
-        }
-    }
-
-    // Null-terminate the argument list
-    argv.push(ptr::null());
-
-    // Use execv to execute the command
-    unsafe { execv(path, argv.as_ptr()) }
-}
-
-/// # Safety
-/// This function is unsafe because it modifies global state.
-#[cfg(all(has_symbol_execlp, has_symbol_execvp))]
-#[unsafe(no_mangle)]
-pub unsafe extern "C" fn execlp(
-    file: *const c_char,
-    arg: *const c_char,
-    args: *const c_char, /* variadic */
-) -> c_int {
-    // Create a vector of argument pointers that we can access
-    let mut argv = Vec::new();
-    argv.push(file);
-
-    if !arg.is_null() {
-        argv.push(arg);
-
-        // In C, execlp() is often implemented with execvp() and just copying the
-        // arguments we have access to
-        let va_arg = args;
-        if !va_arg.is_null() {
-            argv.push(va_arg);
-        }
-    }
-
-    // Null-terminate the argument list
-    argv.push(ptr::null());
-
-    // Use execvp to execute the command
-    unsafe { execvp(file, argv.as_ptr()) }
-}
-
-/// # Safety
-/// This function is unsafe because it modifies global state.
-#[cfg(all(has_symbol_execle, has_symbol_execve))]
-#[unsafe(no_mangle)]
-pub unsafe extern "C" fn execle(
-    path: *const c_char,
-    arg: *const c_char,
-    args: *const c_char, /* variadic */
-) -> c_int {
-    // execle is a variadic function in C that takes arguments followed by an environment pointer
-    // In a real implementation, the last argument (after NULL) is the environment pointer
-
-    // Create a vector of argument pointers that we can access
-    let mut argv = Vec::new();
-    argv.push(path);
-
-    if !arg.is_null() {
-        argv.push(arg);
-
-        // We can only grab the next argument directly
-        let va_arg = args;
-        if !va_arg.is_null() {
-            argv.push(va_arg);
-        }
-    }
-
-    // Null-terminate the argument list
-    argv.push(ptr::null());
-
-    // In execle, the envp pointer follows the NULL terminator
-    // Since we can't access variadic args reliably, we'll use the current environment
-    let current_env = std::env::vars()
-        .map(|(k, v)| format!("{k}={v}"))
-        .map(|s| CString::new(s).unwrap())
-        .collect::<Vec<_>>();
-
-    let mut env_ptrs: Vec<*const c_char> = current_env.iter().map(|cs| cs.as_ptr()).collect();
-
-    env_ptrs.push(ptr::null());
-
-    // Use execve to execute the command
-    unsafe { execve(path, argv.as_ptr(), env_ptrs.as_ptr()) }
-}
-
-/// # Safety
-/// This function is unsafe because it modifies global state.
+/// This function is unsafe because it:
+/// - Dereferences raw pointers
+/// - Calls the real posix_spawn function
 #[cfg(has_symbol_posix_spawn)]
 #[unsafe(no_mangle)]
-pub unsafe extern "C" fn posix_spawn(
+pub unsafe extern "C" fn rust_posix_spawn(
     pid: *mut pid_t,
     path: *const c_char,
     file_actions: *const posix_spawn_file_actions_t,
@@ -485,6 +383,15 @@ pub unsafe extern "C" fn posix_spawn(
     argv: *const *const c_char,
     envp: *const *const c_char,
 ) -> c_int {
+    type PosixSpawnFunc = unsafe extern "C" fn(
+        pid: *mut pid_t,
+        path: *const c_char,
+        file_actions: *const posix_spawn_file_actions_t,
+        attrp: *const posix_spawnattr_t,
+        argv: *const *const c_char,
+        envp: *const *const c_char,
+    ) -> c_int;
+
     unsafe {
         report(|| {
             let result = Execution {
@@ -496,10 +403,14 @@ pub unsafe extern "C" fn posix_spawn(
             Ok(result)
         });
 
+        // Doctor the environment if session is active but the build system stripped preload vars
+        let resolved_env = resolve_environment(envp);
+        let envp_to_use = resolved_env.as_ptr();
+
         let func_ptr = REAL_POSIX_SPAWN.load(Ordering::SeqCst);
         if !func_ptr.is_null() {
             let real_func_ptr: PosixSpawnFunc = std::mem::transmute(func_ptr);
-            real_func_ptr(pid, path, file_actions, attrp, argv, envp)
+            real_func_ptr(pid, path, file_actions, attrp, argv, envp_to_use)
         } else {
             log::error!("Real posix_spawn function not found");
             libc::ENOSYS
@@ -507,11 +418,17 @@ pub unsafe extern "C" fn posix_spawn(
     }
 }
 
+/// Rust implementation of posix_spawnp
+///
+/// Called from C shim for: posix_spawnp
+///
 /// # Safety
-/// This function is unsafe because it modifies global state.
+/// This function is unsafe because it:
+/// - Dereferences raw pointers
+/// - Calls the real posix_spawnp function
 #[cfg(has_symbol_posix_spawnp)]
 #[unsafe(no_mangle)]
-pub unsafe extern "C" fn posix_spawnp(
+pub unsafe extern "C" fn rust_posix_spawnp(
     pid: *mut pid_t,
     file: *const c_char,
     file_actions: *const posix_spawn_file_actions_t,
@@ -519,6 +436,15 @@ pub unsafe extern "C" fn posix_spawnp(
     argv: *const *const c_char,
     envp: *const *const c_char,
 ) -> c_int {
+    type PosixSpawnpFunc = unsafe extern "C" fn(
+        pid: *mut pid_t,
+        file: *const c_char,
+        file_actions: *const posix_spawn_file_actions_t,
+        attrp: *const posix_spawnattr_t,
+        argv: *const *const c_char,
+        envp: *const *const c_char,
+    ) -> c_int;
+
     unsafe {
         report(|| {
             let result = Execution {
@@ -530,10 +456,14 @@ pub unsafe extern "C" fn posix_spawnp(
             Ok(result)
         });
 
+        // Doctor the environment if session is active but the build system stripped preload vars
+        let resolved_env = resolve_environment(envp);
+        let envp_to_use = resolved_env.as_ptr();
+
         let func_ptr = REAL_POSIX_SPAWNP.load(Ordering::SeqCst);
         if !func_ptr.is_null() {
             let real_func_ptr: PosixSpawnpFunc = std::mem::transmute(func_ptr);
-            real_func_ptr(pid, file, file_actions, attrp, argv, envp)
+            real_func_ptr(pid, file, file_actions, attrp, argv, envp_to_use)
         } else {
             log::error!("Real posix_spawnp function not found");
             libc::ENOSYS
@@ -541,7 +471,9 @@ pub unsafe extern "C" fn posix_spawnp(
     }
 }
 
-/// Intercept popen function calls
+/// Rust implementation of popen
+///
+/// Called from C shim for: popen
 ///
 /// # Safety
 /// This function is unsafe because it:
@@ -552,7 +484,9 @@ pub unsafe extern "C" fn posix_spawnp(
 /// The caller must ensure that `command` and `mode` are valid null-terminated C strings.
 #[cfg(has_symbol_popen)]
 #[unsafe(no_mangle)]
-pub unsafe extern "C" fn popen(command: *const c_char, mode: *const c_char) -> *mut libc::FILE {
+pub unsafe extern "C" fn rust_popen(command: *const c_char, mode: *const c_char) -> *mut libc::FILE {
+    type PopenFunc = unsafe extern "C" fn(command: *const c_char, mode: *const c_char) -> *mut libc::FILE;
+
     // For popen, we need to parse the shell command to extract the executable
     if !command.is_null() {
         let command_str = match unsafe { CStr::from_ptr(command) }.to_str() {
@@ -590,7 +524,9 @@ pub unsafe extern "C" fn popen(command: *const c_char, mode: *const c_char) -> *
     }
 }
 
-/// Intercept system function calls
+/// Rust implementation of system
+///
+/// Called from C shim for: system
 ///
 /// # Safety
 /// This function is unsafe because it:
@@ -601,7 +537,9 @@ pub unsafe extern "C" fn popen(command: *const c_char, mode: *const c_char) -> *
 /// The caller must ensure that `command` is a valid null-terminated C string.
 #[cfg(has_symbol_system)]
 #[unsafe(no_mangle)]
-pub unsafe extern "C" fn system(command: *const c_char) -> c_int {
+pub unsafe extern "C" fn rust_system(command: *const c_char) -> c_int {
+    type SystemFunc = unsafe extern "C" fn(command: *const c_char) -> c_int;
+
     // For system, we need to parse the shell command to extract the executable
     if !command.is_null() {
         let command_str = match unsafe { CStr::from_ptr(command) }.to_str() {
@@ -639,15 +577,70 @@ pub unsafe extern "C" fn system(command: *const c_char) -> c_int {
     }
 }
 
-// Function to report a command execution only if reporter is available
+// =============================================================================
+// Helper functions
+// =============================================================================
+
+/// Represents either the original environment pointer or a doctored environment.
+///
+/// This enum allows us to either pass through the original `envp` unchanged,
+/// or use a doctored environment that ensures preload settings are preserved.
+enum ResolvedEnvironment {
+    /// Use the original environment pointer as-is
+    Original(*const *const c_char),
+    /// Use a doctored environment (owns the data)
+    Doctored(DoctoredEnvironment),
+}
+
+impl ResolvedEnvironment {
+    /// Get a pointer to the environment array suitable for passing to exec functions.
+    fn as_ptr(&self) -> *const *const c_char {
+        match self {
+            Self::Original(ptr) => *ptr,
+            Self::Doctored(doc) => doc.as_ptr(),
+        }
+    }
+}
+
+/// Determine whether to use the original environment or create a doctored one.
+///
+/// Logic:
+/// - If SESSION is not set: use original envp (we're not in an interception session)
+/// - If SESSION is set and `in_session` returns true: use original envp (environment is correct)
+/// - If SESSION is set and `in_session` returns false: create doctored environment
+///
+/// # Safety
+/// The `envp` pointer must be a valid null-terminated array of null-terminated
+/// C strings in "KEY=VALUE" format, or null.
+unsafe fn resolve_environment(envp: *const *const c_char) -> ResolvedEnvironment {
+    match SESSION.get() {
+        None => {
+            // No session, pass through original environment
+            ResolvedEnvironment::Original(envp)
+        }
+        Some(state) => {
+            if unsafe { in_session(state.clone(), envp) } {
+                // Environment is still aligned with session, pass through
+                ResolvedEnvironment::Original(envp)
+            } else {
+                // Environment was modified, doctor it
+                match DoctoredEnvironment::from_envp(state.clone(), envp) {
+                    Ok(doctored) => ResolvedEnvironment::Doctored(doctored),
+                    Err(_) => ResolvedEnvironment::Original(envp),
+                }
+            }
+        }
+    }
+}
+
+/// Report a command execution only if reporter is available
 fn report<F>(f: F)
 where
     F: FnOnce() -> Result<Execution, c_int>,
 {
-    let reporter = REPORTER.load(Ordering::SeqCst);
-    if reporter.is_null() {
+    let Some(reporter) = REPORTER.get() else {
         return;
-    }
+    };
 
     let event = match f() {
         Ok(execution) => Event::new(execution),
@@ -657,12 +650,12 @@ where
         }
     };
 
-    if let Err(err) = unsafe { (*reporter).report(event) } {
-        log::debug!("Failed to report execution: {err}",);
+    if let Err(err) = reporter.report(event) {
+        log::debug!("Failed to report execution: {err}");
     }
 }
 
-// Utility functions to convert C arguments to Rust types
+/// Convert a C string to a Rust String
 unsafe fn as_string(s: *const c_char) -> Result<String, c_int> {
     if s.is_null() {
         return Err(libc::EINVAL);
@@ -673,6 +666,7 @@ unsafe fn as_string(s: *const c_char) -> Result<String, c_int> {
     }
 }
 
+/// Convert a C string to a PathBuf
 unsafe fn as_path_buf(s: *const c_char) -> Result<PathBuf, c_int> {
     if s.is_null() {
         return Err(libc::EINVAL);
@@ -683,6 +677,7 @@ unsafe fn as_path_buf(s: *const c_char) -> Result<PathBuf, c_int> {
     }
 }
 
+/// Convert a null-terminated array of C strings to a Vec<String>
 unsafe fn as_string_vec(s: *const *const c_char) -> Result<Vec<String>, c_int> {
     if s.is_null() {
         return Err(libc::EINVAL);
@@ -701,6 +696,7 @@ unsafe fn as_string_vec(s: *const *const c_char) -> Result<Vec<String>, c_int> {
     Ok(vec)
 }
 
+/// Convert a null-terminated array of "KEY=VALUE" C strings to a HashMap
 unsafe fn as_environment(s: *const *const c_char) -> Result<HashMap<String, String>, c_int> {
     if s.is_null() {
         return Err(libc::EINVAL);
@@ -717,7 +713,7 @@ unsafe fn as_environment(s: *const *const c_char) -> Result<HashMap<String, Stri
 
                     map.insert(key, value);
                 }
-                // FIXME: is the `=` always there? Or can be without?
+                // Note: entries without '=' are technically valid but unusual
             }
             Err(e) => return Err(e),
         }
@@ -727,7 +723,31 @@ unsafe fn as_environment(s: *const *const c_char) -> Result<HashMap<String, Stri
     Ok(map)
 }
 
+/// Get the current working directory
 fn working_dir() -> Result<PathBuf, c_int> {
     let cwd = std::env::current_dir().map_err(|_| libc::EINVAL)?;
     Ok(cwd)
 }
+
+/// Set errno to the given value.
+///
+/// Exec-family functions (execve, execvpe, etc.) return -1 on failure and communicate
+/// the error code through errno, not through their return value. This helper provides
+/// portable errno-setting across supported Unix platforms.
+///
+/// # Safety
+/// This directly writes to the thread-local errno location via platform-specific libc functions.
+#[cfg(any(target_os = "linux", target_os = "android"))]
+unsafe fn set_errno(value: c_int) {
+    unsafe {
+        *libc::__errno_location() = value;
+    }
+}
+
+/// Set errno to the given value (macOS / FreeBSD / DragonFly variant).
+#[cfg(any(target_os = "macos", target_os = "ios", target_os = "freebsd", target_os = "dragonfly"))]
+unsafe fn set_errno(value: c_int) {
+    unsafe {
+        *libc::__error() = value;
+    }
+}
